@@ -233,6 +233,7 @@ namespace Arc {
 
     ActivityOldID = j.ActivityOldID;
     LocalInputFiles = j.LocalInputFiles;
+    DelegationID = j.DelegationID;
 
     return *this;
   }
@@ -377,6 +378,13 @@ namespace Arc {
       }
     }
 
+    if (job["Associations"]["DelegationID"]) {
+      DelegationID.clear();
+      for (XMLNode n = job["Associations"]["DelegationID"]; n; ++n) {
+        DelegationID.push_back((std::string)n);
+      }
+    }
+
     // Pick generic GLUE2 information
     SetFromXML(job);
 
@@ -515,7 +523,8 @@ namespace Arc {
     STRINGTOXML(SubmissionClientName)
     STRINGLISTTOXML(OtherMessages)
 
-    if ((ActivityOldID.size() > 0 || LocalInputFiles.size() > 0) && !node["Associations"]) {
+    if ((ActivityOldID.size() > 0 || LocalInputFiles.size() > 0 || DelegationID.size() > 0) &&
+        !node["Associations"]) {
       node.NewChild("Associations");
     }
 
@@ -530,15 +539,23 @@ namespace Arc {
       lif.NewChild("Source") = it->first;
       lif.NewChild("CheckSum") = it->second;
     }
+
+    for (std::list<std::string>::const_iterator it = DelegationID.begin();
+         it != DelegationID.end(); it++) {
+      node["Associations"].NewChild("DelegationID") = *it;
+    }
   }
 
   void Job::SaveToStream(std::ostream& out, bool longlist) const {
     out << IString("Job: %s", JobID) << std::endl;
     if (!Name.empty())
       out << IString(" Name: %s", Name) << std::endl;
-    if (!State().empty())
-      out << IString(" State: %s (%s)", State.GetGeneralState(), State.GetSpecificState())
+    out << IString(" State: %s", State.GetGeneralState())
+              << std::endl;
+    if (longlist && !State().empty()) {
+      out << IString(" Specific state: %s", State.GetSpecificState())
                 << std::endl;
+    }
     if (State == JobState::QUEUING && WaitingPosition != -1) {
       out << IString(" Waiting Position: %d", WaitingPosition) << std::endl;
     }
@@ -633,6 +650,13 @@ namespace Arc {
       if (StageInDir) out << IString(" Stagein directory URL: %s", StageInDir.fullstr()) << std::endl;
       if (StageOutDir) out << IString(" Stageout directory URL: %s", StageOutDir.fullstr()) << std::endl;
       if (SessionDir) out << IString(" Session directory URL: %s", SessionDir.fullstr()) << std::endl;
+      if (!DelegationID.empty()) {
+        out << IString(" Delegation IDs:") << std::endl;
+        for (std::list<std::string>::const_iterator it = DelegationID.begin();
+             it != DelegationID.end(); ++it) {
+          out << "  " << *it << std::endl;
+        }
+      }
     }
 
     out << std::endl;
@@ -699,18 +723,23 @@ namespace Arc {
       return false;
     }
 
-    // We must make it sure it is directory and it exists 
-    if (!DirCreate(dst.Path(), S_IRWXU, true)) {
-      logger.msg(WARNING, "Failed to create directory %s! Skipping job.", dst.Path());
-      return false;
-    }
-
     std::list<std::string> files;
     if (!ListFilesRecursive(uc, src, files)) {
       logger.msg(ERROR, "Unable to retrieve list of job files to download for job %s", JobID);
       return false;
     }
 
+    if (files.empty()) {
+      logger.msg(WARNING, "No files to retrieve for job %s", JobID);
+      return true;
+    }
+
+    // We must make it sure it is directory and it exists 
+    if (!DirCreate(dst.Path(), S_IRWXU, true)) {
+      logger.msg(WARNING, "Failed to create directory %s! Skipping job.", dst.Path());
+      return false;
+    }
+
     const std::string srcpath = src.Path() + (src.Path().empty() || *src.Path().rbegin() != '/' ? "/" : "");
     const std::string dstpath = dst.Path() + (dst.Path().empty() || *dst.Path().rbegin() != G_DIR_SEPARATOR ? G_DIR_SEPARATOR_S : "");
 
