@@ -236,9 +236,13 @@ NS SOAPEnvelope::Namespaces(void) {
 void SOAPEnvelope::GetXML(std::string& out_xml_str,bool user_friendly) const {
   if(header.Size() == 0) {
     SOAPEnvelope& it = *(SOAPEnvelope*)this;
-    it.header.Destroy();
+    // Moving header outside tree and then back.
+    // This fully recovers initial tree and allows this methof to be const
+    XMLNode tmp_header;
+    it.header.Move(tmp_header);
     envelope.GetXML(out_xml_str,user_friendly);
-    it.header=it.envelope.NewChild("soap-env:Header",0,true);
+    it.header=it.envelope.NewChild("soap-env:Header",0,true); // It can be any dummy
+    it.header.Exchange(tmp_header);
     return;
   };
   envelope.GetXML(out_xml_str,user_friendly);
@@ -301,6 +305,7 @@ std::string SOAPFault::Reason(int num) {
 }
 
 void SOAPFault::Reason(int num,const char* r) {
+  if(!r) r = "";
   if(ver12) {
     if(!reason) reason=fault.NewChild(fault.Prefix()+":Reason");
     XMLNode rn = reason.Child(num);
@@ -309,7 +314,13 @@ void SOAPFault::Reason(int num,const char* r) {
     return;
   };
   if(!reason) reason=fault.NewChild(fault.Prefix()+":faultstring");
-  reason=r;
+  if(*r) {
+    reason=r;
+  } else {
+    // RFC says it SHOULD provide some description.
+    // And some implementations take it too literally.
+    reason="unknown";
+  };
   return;
 }
 
@@ -318,6 +329,7 @@ std::string SOAPFault::Node(void) {
 }
 
 void SOAPFault::Node(const char* n) {
+  if(!n) n = "";
   if(!node) {
     if(ver12) {
       node=fault.NewChild(fault.Prefix()+":Node");
@@ -333,6 +345,7 @@ std::string SOAPFault::Role(void) {
 }
 
 void SOAPFault::Role(const char* r) {
+  if(!r) r = "";
   if(ver12) {
     if(!role) role=fault.NewChild(fault.Prefix()+":Role");
     role=r;
@@ -340,6 +353,8 @@ void SOAPFault::Role(const char* r) {
 }
 
 static const char* FaultCodeMatch(const char* base,const char* code) {
+  if(!base) base = "";
+  if(!code) code = "";
   int l = strlen(base);
   if(strncasecmp(base,code,l) != 0) return NULL;
   if(code[l] == 0) return code+l;
@@ -376,8 +391,8 @@ SOAPFault::SOAPFaultCode SOAPFault::Code(void) {
       return Sender;
     if(FaultCodeMatch("Server",c.c_str())) 
       return Receiver;
-    return unknown;
   };
+  return unknown;
 }
 
 void SOAPFault::Code(SOAPFaultCode c) {
@@ -391,7 +406,7 @@ void SOAPFault::Code(SOAPFaultCode c) {
       case DataEncodingUnknown: value=value.Prefix()+":DataEncodingUnknown"; break;
       case Sender: value=value.Prefix()+":Sender"; break;
       case Receiver: value=value.Prefix()+":Receiver"; break;
-      default: value="";
+      default: value=""; break;
     };
   } else {
     if(!code) code=fault.NewChild(fault.Prefix()+":faultcode");
@@ -400,7 +415,7 @@ void SOAPFault::Code(SOAPFaultCode c) {
       case MustUnderstand: code=code.Prefix()+":MustUnderstand"; break;
       case Sender: code=code.Prefix()+":Client"; break;
       case Receiver: code=code.Prefix()+":Server"; break;
-      default: code="";
+      default: code=""; break;
     };
   };
 }
@@ -420,6 +435,7 @@ std::string SOAPFault::Subcode(int level) {
 void SOAPFault::Subcode(int level,const char* s) {
   if(!ver12) return;
   if(level < 0) return;
+  if(!s) s = "";
   if(!code) code=fault.NewChild(fault.Prefix()+":Code");
   XMLNode subcode = code;
   for(;level;--level) {
