@@ -120,6 +120,8 @@ namespace DataStaging {
     delivery_pair_t* dp = (delivery_pair_t*)arg;
     delete dp->comm;
     dp->comm = NULL;
+    // In case transfer finished before getting cancel signal, delete destination
+    if (dp->cancelled || dp->dtr->error()) dp->dtr->get_destination()->Remove();
   }
 
   bool DataDelivery::delete_delivery_pair(delivery_pair_t* dp) {
@@ -131,6 +133,22 @@ namespace DataStaging {
     return res;
   }
 
+  // Delete DTR destination, called after losing contact with delivery process
+  static void delete_dtr_destination(DTR_ptr dtr) {
+    Arc::URL dest(dtr->get_destination()->CurrentLocation());
+    // Check for TURL
+    if (!dtr->get_destination()->TransferLocations().empty()) {
+      dest = dtr->get_destination()->TransferLocations().front();
+    }
+    // Check for cache file
+    if ((dtr->get_cache_state() == CACHEABLE) && !dtr->get_cache_file().empty()) {
+      dest = dtr->get_cache_file();
+    }
+    dtr->get_logger()->msg(Arc::VERBOSE, "Cleaning up after failure: deleting %s", dest.str());
+    Arc::DataHandle h(dest, dtr->get_usercfg());
+    if (h) h->Remove();
+  }
+
   void DataDelivery::main_thread (void* arg) {
     DataDelivery* it = (DataDelivery*)arg;
     it->main_thread();
@@ -141,6 +159,7 @@ namespace DataStaging {
     // messages are logged to per-DTR Logger
     Arc::Logger::getRootLogger().setThreadContext();
     Arc::Logger::getRootLogger().removeDestinations();
+    Arc::Logger::getRootLogger().setThreshold(DTR::LOG_LEVEL);
 
     while(delivery_state != TO_STOP){
       dtr_list_lock.lock();
@@ -166,8 +185,7 @@ namespace DataStaging {
           // With ThreadedPointer this may not be a problem any more.
           DTR_ptr tmp = dp->dtr;
           if (!delete_delivery_pair(dp)) {
-            tmp->get_logger()->msg(Arc::ERROR, "DTR %s: Failed to delete delivery object or deletion timed out",
-                                                  tmp->get_short_id());
+            tmp->get_logger()->msg(Arc::ERROR, "Failed to delete delivery object or deletion timed out");
           }
           tmp->set_status(DTRStatus::TRANSFERRED);
           DTR::push(tmp, SCHEDULER);
@@ -212,26 +230,29 @@ namespace DataStaging {
            (status.commstatus == DataDeliveryComm::CommClosed) ||
            (status.commstatus == DataDeliveryComm::CommFailed)) {
           // Transfer finished - either successfully or with error
-          // comm.GetError()
           dtr_list_lock.lock();
           d = dtr_list.erase(d);
           dtr_list_lock.unlock();
-          if((status.commstatus == DataDeliveryComm::CommFailed) ||
-             (status.error != DTRErrorStatus::NONE_ERROR)) {
-            if(status.error == DTRErrorStatus::NONE_ERROR)
+
+          if ((status.commstatus == DataDeliveryComm::CommFailed) ||
+              (status.error != DTRErrorStatus::NONE_ERROR)) {
+
+            if (status.error == DTRErrorStatus::NONE_ERROR) {
+              // Lost track of process - delete destination so it can be tried again
+              delete_dtr_destination(dp->dtr);
               status.error = DTRErrorStatus::INTERNAL_PROCESS_ERROR;
+            }
             dp->dtr->set_error_status(status.error,status.error_location,
                      status.error_desc[0]?status.error_desc:dp->comm->GetError().c_str());
-          } else if (status.checksum) {
+          }
+          else if (status.checksum) {
             dp->dtr->get_destination()->SetCheckSum(status.checksum);
           }
-          dp->dtr->get_logger()->msg(Arc::INFO, "DTR %s: Transfer finished: %llu bytes transferred %s",
-                                     dp->dtr->get_short_id(), status.transferred,
+          dp->dtr->get_logger()->msg(Arc::INFO, "Transfer finished: %llu bytes transferred %s", status.transferred,
                                      (status.checksum[0] ? ": checksum "+std::string(status.checksum) : " "));
           DTR_ptr tmp = dp->dtr;
           if (!delete_delivery_pair(dp)) {
-            tmp->get_logger()->msg(Arc::ERROR, "DTR %s: Failed to delete delivery object or deletion timed out",
-                                                  tmp->get_short_id());
+            tmp->get_logger()->msg(Arc::ERROR, "Failed to delete delivery object or deletion timed out");
           }
           tmp->set_status(DTRStatus::TRANSFERRED);
           DTR::push(tmp, SCHEDULER);
@@ -262,14 +283,15 @@ namespace DataStaging {
           }
           else {
             if (comm_err.empty()) comm_err = "Connection with delivery process lost";
+            // delete destination so it can be tried again
+            delete_dtr_destination(dp->dtr);
             dp->dtr->set_error_status(DTRErrorStatus::INTERNAL_PROCESS_ERROR,
                                       DTRErrorStatus::ERROR_TRANSFER,
                                       comm_err);
           }
           DTR_ptr tmp = dp->dtr;
           if (!delete_delivery_pair(dp)) {
-            tmp->get_logger()->msg(Arc::ERROR, "DTR %s: Failed to delete delivery object or deletion timed out",
-                                                  tmp->get_short_id());
+            tmp->get_logger()->msg(Arc::ERROR, "Failed to delete delivery object or deletion timed out");
           }
           tmp->set_status(DTRStatus::TRANSFERRED);
           DTR::push(tmp, SCHEDULER);
@@ -288,8 +310,7 @@ namespace DataStaging {
     for (std::list<delivery_pair_t*>::iterator d = dtr_list.begin(); d != dtr_list.end();) {
       DTR_ptr tmp = (*d)->dtr;
       if (!delete_delivery_pair(*d)) {
-        tmp->get_logger()->msg(Arc::ERROR, "DTR %s: Failed to delete delivery object or deletion timed out",
-                                              tmp->get_short_id());
+        tmp->get_logger()->msg(Arc::ERROR, "Failed to delete delivery object or deletion timed out");
       }
       d = dtr_list.erase(d);
     }
