@@ -26,6 +26,7 @@
 namespace ARex {
 
 Arc::Logger CoreConfig::logger(Arc::Logger::getRootLogger(), "CoreConfig");
+#define REPORTER_PERIOD "3600";
 
 void CoreConfig::CheckLRMSBackends(const std::string& default_lrms) {
   std::string tool_path;
@@ -195,7 +196,7 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
         config.job_log->SetVoFilters(voFilters.c_str());
       }
     }
-    else if (command == "jobreport_publisher") { // Name of the publisher: e.g. jura, arc-ur-logger
+    else if (command == "jobreport_publisher") { // Name of the publisher: e.g. jura
       if (!config.job_log) continue;
       std::string publisher = config_next_arg(rest);
       if (publisher.empty()) {
@@ -203,17 +204,40 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       }
       config.job_log->SetLogger(publisher.c_str());
     }
+    else if (command == "jobreport_period") { // Period of running in seconds: e.g. 3600
+      if (!config.job_log) continue;
+      std::string period_s = config_next_arg(rest);
+      if (period_s.empty()) {
+        period_s = REPORTER_PERIOD;
+      }
+      int period;
+      if (!Arc::stringto(period_s, period)) {
+        logger.msg(Arc::ERROR, "Wrong number in jobreport_period: %s", period_s); return false;
+      }
+      if (!config.job_log->SetPeriod(period)) {
+        std::string default_value = REPORTER_PERIOD;
+        logger.msg(Arc::ERROR, "Wrong number in jobreport_period: %d, minimal value: %s", period, default_value); return false;
+      }
+    }
     else if (command == "jobreport_credentials") {
       if (!config.job_log) continue;
       std::string jobreport_key = config_next_arg(rest);
       std::string jobreport_cert = config_next_arg(rest);
       std::string jobreport_cadir = config_next_arg(rest);
-      config.job_log->set_credentials(jobreport_key, jobreport_cert, jobreport_cadir);
+      config.job_log->SetCredentials(jobreport_key, jobreport_cert, jobreport_cadir);
     }
     else if (command == "jobreport_options") { // e.g. for SGAS, interpreted by usage reporter
       if (!config.job_log) continue;
       std::string accounting_options = config_next_arg(rest);
-      config.job_log->set_options(accounting_options);
+      config.job_log->SetOptions(accounting_options);
+    }
+    else if (command == "jobreport_logfile") {
+      if (!config.job_log) continue;
+      std::string logfile = config_next_arg(rest);
+      if (logfile.empty()) {
+        logger.msg(Arc::ERROR, "Missing file name in jobreport_logfile"); return false;
+      }
+      config.job_log->SetLogFile(logfile.c_str());
     }
     else if (command == "scratchdir") {
       std::string scratch = config_next_arg(rest);
@@ -253,56 +277,15 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       }
       if (config.max_jobs_total < 0) config.max_jobs_total = -1;
     }
-    else if (command == "speedcontrol") {
-      std::string speed_s = config_next_arg(rest);
-      if (!Arc::stringto(speed_s, config.min_speed)) {
-        logger.msg(Arc::ERROR, "Wrong number in speedcontrol: %s", speed_s); return false;
-      }
-      speed_s = config_next_arg(rest);
-      if (!Arc::stringto(speed_s, config.min_speed_time)) {
-        logger.msg(Arc::ERROR, "Wrong number in speedcontrol: %s", speed_s); return false;
-      }
-      speed_s = config_next_arg(rest);
-      if (!Arc::stringto(speed_s, config.min_average_speed)) {
-        logger.msg(Arc::ERROR, "Wrong number in speedcontrol: %s", speed_s); return false;
-      }
-      speed_s = config_next_arg(rest);
-      if (!Arc::stringto(speed_s, config.max_inactivity_time)) {
-        logger.msg(Arc::ERROR, "Wrong number in speedcontrol: %s", speed_s); return false;
-      }
-    }
     else if (command == "wakeupperiod") {
       std::string wakeup_s = config_next_arg(rest);
       if (!Arc::stringto(wakeup_s, config.wakeup_period)) {
         logger.msg(Arc::ERROR,"Wrong number in wakeupperiod: %s",wakeup_s); return false;
       }
     }
-    else if (command == "securetransfer") {
-      if (!CheckYesNoCommand(config.use_secure_transfer, command, rest)) return false;
-    }
-    else if(command == "passivetransfer") {
-      if (!CheckYesNoCommand(config.use_passive_transfer, command, rest)) return false;
-    }
-    else if (command == "maxtransfertries") {
-      std::string maxtries_s = config_next_arg(rest);
-      if (!Arc::stringto(maxtries_s, config.max_retries)) {
-        logger.msg(Arc::ERROR, "Wrong number in maxtransfertries"); return false;
-      }
-    }
-    else if (command == "acix_endpoint")  {
-      std::string endpoint(config_next_arg(rest));
-      if (!Arc::URL(endpoint) || endpoint.find("://") == std::string::npos) {
-        logger.msg(Arc::ERROR, "Bad URL in acix_endpoint"); return false;
-      }
-      endpoint.replace(0, endpoint.find("://"), "acix");
-      config.acix_endpoint = endpoint;
-    }
     else if(command == "norootpower") {
       if (!CheckYesNoCommand(config.strict_session, command, rest)) return false;
     }
-    else if (command == "localtransfer") {
-      if (!CheckYesNoCommand(config.use_local_transfer, command, rest)) return false;
-    }
     else if (command == "mail") { // internal address from which to send mail
       config.support_email_address = config_next_arg(rest);
       if (config.support_email_address.empty()) {
@@ -360,10 +343,6 @@ bool CoreConfig::ParseConfINI(GMConfig& config, std::ifstream& cfile) {
       }
       config.cred_plugin->timeout(timeout);
     }
-    else if (command == "preferredpattern") {
-      std::string preferred_pattern = config_next_arg(rest);
-      config.preferred_pattern = preferred_pattern;
-    }
     else if (command == "fixdirectories") {
       std::string s = config_next_arg(rest);
       if (s == "yes") {
@@ -478,6 +457,7 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     KeyPath
     CertificatePath
     CACertificatesDir
+    logfile
   */
   tmp_node = cfg["enableARCInterface"];
   if (tmp_node) {
@@ -511,12 +491,24 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
       config.job_log->SetLogger(publisher.c_str());
       unsigned int i;
       if (Arc::stringto(tmp_node["expiration"], i)) config.job_log->SetExpiration(i);
+      std::string period = tmp_node["Period"];
+      if (period.empty()) period = REPORTER_PERIOD;
+      unsigned int p;
+      if (!Arc::stringto(period, p)) {
+        logger.msg(Arc::ERROR, "Wrong number in jobreport_period: %s", period); return false;
+      }
+      if (!config.job_log->SetPeriod(p)) {
+        std::string default_value = REPORTER_PERIOD;
+        logger.msg(Arc::ERROR, "Wrong number in jobreport_period: %d, minimal value: %s", p, default_value); return false;
+      }      
       std::string parameters = tmp_node["parameters"];
-      if (!parameters.empty()) config.job_log->set_options(parameters);
+      if (!parameters.empty()) config.job_log->SetOptions(parameters);
       std::string jobreport_key = tmp_node["KeyPath"];
       std::string jobreport_cert = tmp_node["CertificatePath"];
       std::string jobreport_cadir = tmp_node["CACertificatesDir"];
-      config.job_log->set_credentials(jobreport_key, jobreport_cert, jobreport_cadir);
+      config.job_log->SetCredentials(jobreport_key, jobreport_cert, jobreport_cadir);
+      std::string logfile = tmp_node["logfile"];
+      if (!logfile.empty()) config.job_log->SetLogFile(logfile.c_str());
     }
   }
 
@@ -525,11 +517,7 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     maxJobsTracked
     maxJobsRun
     maxJobsTotal
-    maxJobsTransferred
-    maxJobsTransferredAdditional
-    maxFilesTransferred
-    maxLoadShare
-    loadShareType
+    maxJobsPerDN
     wakeupPeriod
   */
   tmp_node = cfg["loadLimits"];
@@ -538,74 +526,9 @@ bool CoreConfig::ParseConfXML(GMConfig& config, const Arc::XMLNode& cfg) {
     if (!elementtoint(tmp_node, "maxJobsRun", config.max_jobs_running, &logger)) return false;
     if (!elementtoint(tmp_node, "maxJobsTotal", config.max_jobs_total, &logger)) return false;
     if (!elementtoint(tmp_node, "maxJobsPerDN", config.max_jobs_per_dn, &logger)) return false;
-    if (!elementtoint(tmp_node, "maxJobsTransferred", config.max_jobs_staging, &logger)) return false;
-    if (!elementtoint(tmp_node, "maxJobsTransferredAdditional", config.max_jobs_staging_emergency, &logger)) return false;
-    if (!elementtoint(tmp_node, "maxFilesTransferred", config.max_downloads, &logger)) return false;
-    std::string transfer_share = tmp_node["loadShareType"];
-    int max_share;
-    if (elementtoint(tmp_node, "maxLoadShare", max_share, &logger) && (max_share > 0) && ! transfer_share.empty()){
-      config.share_type = transfer_share;
-      config.max_staging_share = max_share;
-    }
     if (!elementtoint(tmp_node, "wakeupPeriod", config.wakeup_period, &logger)) return false;
-    Arc::XMLNode share_limit_node = tmp_node["shareLimit"];
-    for (; share_limit_node; ++share_limit_node) {
-      int share_limit = -1;
-      std::string limited_share = share_limit_node["name"];
-      if (elementtoint(share_limit_node, "limit", share_limit, &logger) && (share_limit > 0) && ! limited_share.empty()) {
-        config.limited_share[limited_share] = share_limit;
-      }
-    }
   }
 
-  /*
-  dataTransfer
-    secureTransfer
-    passiveTransfer
-    localTransfer
-    preferredPattern
-    acixEndpoint
-    timeouts
-      minSpeed
-      minSpeedTime
-      minAverageSpeed
-      maxInactivityTime
-    maxRetries
-    mapURL (link)
-      from
-      to
-    Globus
-      gridmapfile
-      CACertificatesDir
-      CertificatePath
-      KeyPath
-      TCPPortRange
-      UDPPortRange
-    httpProxy
-  */
-  tmp_node = cfg["dataTransfer"];
-  if (tmp_node) {
-    Arc::XMLNode to_node = tmp_node["timeouts"];
-    if (to_node) {
-      if (!elementtoint(tmp_node, "minSpeed", config.min_speed, &logger)) return false;
-      if (!elementtoint(tmp_node, "minAverageSpeed", config.min_average_speed, &logger)) return false;
-      if (!elementtoint(tmp_node, "minSpeedTime", config.min_speed_time, &logger)) return false;
-      if (!elementtoint(tmp_node, "maxInactivityTime", config.max_inactivity_time, &logger)) return false;
-    }
-    if (!elementtobool(tmp_node, "passiveTransfer", config.use_passive_transfer, &logger)) return false;
-    if (!elementtobool(tmp_node, "secureTransfer", config.use_secure_transfer, &logger)) return false;
-    if (!elementtobool(tmp_node, "localTransfer", config.use_local_transfer, &logger)) return false;
-    if (!elementtoint(tmp_node, "maxRetries", config.max_retries, &logger)) return false;
-    if (tmp_node["preferredPattern"]) config.preferred_pattern = (std::string)(tmp_node["preferredPattern"]);
-    if (tmp_node["acixEndpoint"]) {
-      std::string endpoint((std::string)(tmp_node["acixEndPoint"]));
-      if (!Arc::URL(endpoint) || endpoint.find("://") == std::string::npos) {
-        logger.msg(Arc::ERROR, "Bad URL in acix_endpoint"); return false;
-      }
-      endpoint.replace(0, endpoint.find("://"), "acix");
-      config.acix_endpoint = endpoint;
-    }
-  }
   /*
   serviceMail
   */
