@@ -260,7 +260,13 @@ namespace Arc
         //NOTE! Current LUTS also sets a "creationTime"[sic!] for each record
         
         // ID for record
-        if (!mainnode.empty())
+        if (find("headnode")!=end()) {
+          Arc::URL headnode((*this)["headnode"]);
+          rid.NewAttribute("urf:recordId")=
+            std::string(recordid_prefix) + headnode.Host() + 
+            '-' + (*this)["ngjobid"]; 
+        }
+        else  if (!mainnode.empty())
           rid.NewAttribute("urf:recordId")=
             std::string(recordid_prefix) + mainnode + 
             '-' + (*this)["ngjobid"];
@@ -295,11 +301,12 @@ namespace Arc
     //NOTE! original JARM used "GlobalUserId"
     if (find("usersn")!=end())
       {
+        ur.NewChild("UserIdentity");
         if (find("localuser")!=end())
           ur["UserIdentity"].NewChild("LocalUserId")=
             (*this)["localuser"];
 
-        ur.NewChild("UserIdentity").NewChild("GlobalUserName")=
+        ur["UserIdentity"].NewChild("GlobalUserName")=
           (*this)["usersn"];
       }
     
@@ -353,19 +360,72 @@ namespace Arc
                     vo_attr.NewChild("vo:Role")=str;
                   }
                   if((pos = attr.find("Capability=")) != std::string::npos) {
-                    std::string str = attr.substr(pos+5);
+                    std::string str = attr.substr(pos+11);
                     vo_attr.NewChild("vo:Capability")=str;
                   }
                 }
             }
         }
-        if ( !needToSend ) {
+        if ( !needToSend && (int)voms_attributes.size() > 0) {
             Arc::Logger::rootLogger.msg(Arc::INFO,
                  "[VO filter] Job log will be not send. %s.", filename.c_str());
             usagerecord.Destroy();
             return;
         }
       }
+
+    if (find("jobreport_option_vomsless_vo")!=end() && !ur["UserIdentity"]["vo:VO"])
+      {
+        if ( !ur["UserIdentity"] ) {
+          ur.NewChild("UserIdentity");
+        }
+        Arc::XMLNode vo=ur["UserIdentity"].NewChild("vo:VO");
+        vo.NewAttribute("vo:type")="voms";
+
+        std::string option=(*this)["jobreport_option_vomsless_vo"];
+        std::string name=option;
+        std::string issuer=option;
+        // separate opt_vo_name#opt_vo_issuer pair
+        pcolon=option.find('#');
+        if (pcolon!=std::string::npos)
+          {
+            name=option.substr(0, pcolon), 
+            issuer=option.substr(pcolon+1, std::string::npos);
+          }
+        vo.NewChild("vo:Name")=name;
+        vo.NewChild("vo:Issuer")=issuer;
+        vo.NewChild("vo:Attribute").NewChild("vo:Group")="/"+name;
+      }
+
+    if (find("jobreport_option_vo_group")!=end())
+      {
+        bool find = false;
+        if ( !ur["UserIdentity"] ) {
+          ur.NewChild("UserIdentity");
+        }
+        Arc::XMLNode useridentity = ur["UserIdentity"];
+
+        Arc::XMLNode node = useridentity["vo:VO"]["vo:Attribute"]["vo:Group"];
+        while ( bool(node) ){
+          if (std::string(node) == (*this)["jobreport_option_vo_group"]) {
+            find = true;
+            break;
+          }
+          ++node;
+        }
+        if ( !find ) {
+          if ( !useridentity["vo:VO"] ) {
+            Arc::XMLNode vo = useridentity.NewChild("vo:VO");
+            vo.NewAttribute("vo:type")="voms";
+          }
+          if ( !useridentity["vo:VO"]["vo:Attribute"] ) {
+            useridentity["vo:VO"].NewChild("vo:Attribute");
+          }
+          useridentity["vo:VO"]["vo:Attribute"].NewChild("vo:Group")=(*this)["jobreport_option_vo_group"].substr(1);
+        }
+      }
+
+      
     
     //JobName
     if (find("jobname")!=end())
@@ -714,7 +774,13 @@ namespace Arc
         rid.NewAttribute("urf:createTime")=nowstamp;
 
         // ID for record
-        if (!mainnode.empty())
+        if (find("headnode")!=end()){
+          Arc::URL headnode((*this)["headnode"]);
+          rid.NewAttribute("urf:recordId")=
+            std::string(recordid_prefix) + headnode.Host() + 
+            '-' + (*this)["ngjobid"]; 
+        }      
+        else if (!mainnode.empty())
           rid.NewAttribute("urf:recordId")=
             std::string(recordid_prefix) + mainnode + 
             '-' + (*this)["ngjobid"];
@@ -806,6 +872,49 @@ namespace Arc
         }
       }
 
+    if (find("jobreport_option_vomsless_vo")!=end() && !useridentity["Group"])
+      {
+        if ( !useridentity["Group"] ) {
+          useridentity.NewChild("Group");
+        }
+        Arc::XMLNode vo=useridentity["Group"];
+
+        std::string option=(*this)["jobreport_option_vomsless_vo"];
+        std::string name=option;
+        std::string issuer=option;
+        // separate opt_vo_name#opt_vo_issuer pair
+        pcolon=option.find('#');
+        if (pcolon!=std::string::npos)
+          {
+            name=option.substr(0, pcolon), 
+            issuer=option.substr(pcolon+1, std::string::npos);
+          }
+        vo=name;
+        Arc::XMLNode gattr=useridentity.NewChild("GroupAttribute")="/"+name;
+        gattr.NewAttribute("urf:type")="vo-role";
+      }
+
+    if (find("jobreport_option_vo_group")!=end())
+      {
+        bool find = false;
+        Arc::XMLNode node = useridentity["GroupAttribute"];
+        while ( bool(node) ){
+          if (node.Attribute("urf:type") == "vo-group" &&
+              std::string(node) == (*this)["jobreport_option_vo_group"]) {
+            find = true;
+            break;
+          }
+          ++node;
+        }
+        if ( !find ) {
+          if ( !useridentity["Group"] ) {
+            useridentity.NewChild("Group")=(*this)["jobreport_option_vo_group"].substr(1);
+          }
+          Arc::XMLNode vog=useridentity.NewChild("GroupAttribute")=(*this)["jobreport_option_vo_group"];
+          vog.NewAttribute("urf:type")="vo-group";
+        }
+      }
+
     if (find("projectname")!=end())
       {
         Arc::XMLNode project=useridentity.NewChild("GroupAttribute")=(*this)["projectname"];
@@ -931,6 +1040,14 @@ namespace Arc
         Arc::XMLNode alln = ur.NewChild("CpuDuration")=cpudurs;
         alln.NewAttribute("urf:usageType")="all";
       }
+    else
+      {
+        Arc::Logger::rootLogger.msg(Arc::VERBOSE,
+                        "Missing required element \"CpuDuration\" in job log file %s",
+                        filename.c_str());
+        usagerecord.Destroy();
+        return;
+      }
 
     //ServiceLevel
     std::string benchmark_type = "Si2K";
